//LICENSE A


#ifndef IPOINTER_H
#define IPOINTER_H


//
//  various simple but useful pointer types
//
namespace iPointer
{
	//
	//  helper function
	//
	void Validate(void *ptr);


	//
	//  Small common functionality
	//
	template<class T>
	class TrivialBase
	{

	public:

		//
		// Cast to the holder's type
		//
		inline operator T*() const { return this->mPtr; }

		//
		// Dereference the pointer and return a reference to the contained object.
		//
		inline T& operator*() const { return *this->mPtr; }

		//
		// Provides normal pointer target member access using operator ->.
		//
		inline T* operator->() const { return this->mPtr; }

	protected:

		TrivialBase(T* ptr)
		{
			this->mPtr = ptr;
		}

		T* mPtr;

	private:

		//
		//  Not implemented
		//
		TrivialBase(); // cannot have an automatic default constructor - otherwise the compiler will not require explicit instantination
		TrivialBase& operator=(TrivialBase&);
	};


	//
	//  Simple non-assignable, non-transferable auto-ptr
	//
	template<class T>
	class AutoDeleted : public TrivialBase<T>
	{

	public:

		template<class P>
		AutoDeleted(P parent) : TrivialBase<T>(T::New(parent))
		{
			Validate(this->mPtr);
		}

		AutoDeleted() : TrivialBase<T>(T::New())
		{
			Validate(this->mPtr);
		}

		~AutoDeleted()
		{
			this->mPtr->Delete();
		}

	private:

		//
		//  Not implemented
		//
		//AutoDeleted(); // cannot have an automatic default constructor - otherwise the compiler will not require explicit instantination
		AutoDeleted& operator=(AutoDeleted&);
	};


	//
	//  A pointer that cannot have a NULL value
	//
	template<class T>
	class AlwaysValid : public TrivialBase<T>
	{

	public:

		AlwaysValid(T *ptr) : TrivialBase<T>(ptr)
		{
			Validate(this->mPtr);
		}

	private:

		//
		//  Not implemented
		//
		AlwaysValid();
		AlwaysValid& operator=(AlwaysValid&);
	};


	//
	//  A pointer ordered by GetId() call of the argument class
	//
	template<class T>
	class Ordered : public TrivialBase<T>
	{

	public:

		Ordered(T *ptr) : TrivialBase<T>(ptr)
		{ 
		}

		//
		//  Allow assignment operator
		//
		void operator=(const Ordered<T> &p)
		{
			this->mPtr = p.mPtr;
		}

		bool operator==(const Ordered<T> &other) const
		{
			return (this->mPtr==0 ? other.mPtr==0 : (other.mPtr!=0 && this->mPtr->GetId()==other.mPtr->GetId()));
		}

		bool operator<(const Ordered<T> &other) const
		{
			return (this->mPtr==0 ? other.mPtr!=0 : (other.mPtr!=0 && this->mPtr->GetId()<other.mPtr->GetId()));
		}

	private:

		//
		//  Not implemented
		//
		Ordered();
	};
};

#endif // IPOINTER_H

